
#include "Arduino.h"

#define IS_READY 100
#define SET_FILTRAGE_32BITS 103
#define SET_FILTRAGE_FLOAT 104

#define GAIN 1.45
#define SAT
#define TTLOUT 

// ADC
#define ADC_MR_TRIG1 (1<<1)
uint8_t nchannels;
#define MAX_NCHAN 2
uint8_t channels[MAX_NCHAN];
uint8_t gain[MAX_NCHAN];
uint8_t channels_pins[8] ={7,6,5,4,3,2,1,0};
uint32_t ticks;
volatile void (*ADC_Handler_function)();
uint8_t flag = 0;

// ADC
uint32_t chsel;

//filtrage
#define MAX_NCOEF 256
uint32_t nb; // nombre de coefficients b
float bbb[MAX_NCOEF];
int32_t bb[MAX_NCOEF];
uint32_t na; // nombre de coefficients a
float aaa[MAX_NCOEF];
int32_t aa[MAX_NCOEF];
uint32_t max_na_nb;
uint8_t ab_shift;
#define TAILLE_TAMPON 0x100 // supérieur à max(na,nb)
#define MASQUE_TAMPON 0xFF // TAILLE_TAMPON-1
int16_t x_0[TAILLE_TAMPON]; // tampon pour les x_k
int16_t y_0[TAILLE_TAMPON]; // tampon pour les y_k
int16_t x_1[TAILLE_TAMPON]; // tampon pour les x_k
int16_t y_1[TAILLE_TAMPON]; // tampon pour les y_k
int32_t ww_0[TAILLE_TAMPON];
int32_t ww_1[TAILLE_TAMPON];
float www_0[TAILLE_TAMPON];
float www_1[TAILLE_TAMPON];
uint8_t J_tampon; // indice pour les tampons
int16_t offset;
             
void config_adc() {
    int k;
    pmc_enable_periph_clk(ID_ADC);
    adc_init(ADC, VARIANT_MCK, ADC_FREQ_MAX, 0);
    adc_configure_timing(ADC, 0, ADC_SETTLING_TIME_3, 1);
    adc_set_resolution(ADC,ADC_12_BITS);
    NVIC_EnableIRQ (ADC_IRQn) ;   
    ADC->ADC_IDR=0xFFFFFFFF ;  
    ADC->ADC_CHDR = 0xFFFF; 
    ADC->ADC_CHSR = 0x0000; 
    ADC->ADC_WPMR &= ~0x1;
    ADC->ADC_CHER = 0x0000;
    ADC->ADC_SEQR1 = 0x00000000;
    ADC->ADC_CGR = 0x00000000;
    ADC->ADC_COR = 0x00000000;
    for (k=0; k<nchannels; k++) {
      ADC->ADC_SEQR1 |= ((channels[k]&0xF)<<(4*k));
      ADC->ADC_CHER |= (1<<k);
      if (gain[k]==2) ADC->ADC_CGR |= 2 << (2*k);
      if (gain[k]==4) ADC->ADC_CGR |= 3 << (2*k);
      ADC->ADC_COR |= 1 << k; 
    }
    ADC->ADC_MR = (ADC->ADC_MR & 0xFFFFFFF0) | ADC_MR_TRIG1 | ADC_MR_TRGEN | ADC_MR_USEQ;
    ADC->ADC_IDR=~(1<<channels[0]); 
    ADC->ADC_IER=1<<channels[0];
}
                
void ADC_Handler() {
    (*ADC_Handler_function)();
#ifdef TTLOUT
  if (flag==0) {
      PIOC->PIO_SODR = PIO_PC28; // pin 3
      flag = 1;
    }
  else {
      PIOC->PIO_CODR = PIO_PC28;
      flag = 0;
   }
#endif
}
                 
void config_timer(uint32_t ticks) {
  uint32_t channel = 0;
  uint8_t clock = TC_CMR_TCCLKS_TIMER_CLOCK1; // horloge 84MHz/2=42 MHz
  pmc_enable_periph_clk (TC_INTERFACE_ID + 0*3+channel) ;
  TC_Configure(TC0,channel, TC_CMR_WAVE | TC_CMR_WAVSEL_UP_RC | TC_CMR_ACPA_CLEAR | TC_CMR_ACPC_SET |clock); 
  TC_SetRC(TC0,channel,ticks); 
  TC_SetRA(TC0,channel,ticks >> 1);
  TC_Start(TC0, channel);
}
void stop_timer() {
    uint32_t channel = 0;
    TC_Stop(TC0,channel);
    pmc_disable_periph_clk (TC_INTERFACE_ID + 0*3+channel);
}
                
void config_dac(uint32_t channel) {
   pmc_enable_periph_clk(DACC_INTERFACE_ID);
   dacc_reset(DACC_INTERFACE);
   dacc_set_transfer_mode(DACC_INTERFACE, 0); // half word transfert (16 bits)
   dacc_set_power_save(DACC_INTERFACE, 0, 0);
   dacc_set_timing(DACC_INTERFACE, 0x08, 0, 0x10);
   dacc_set_analog_control(DACC_INTERFACE, DACC_ACR_IBCTLCH0(0x02)|DACC_ACR_IBCTLCH1(0x02)|DACC_ACR_IBCTLDACCORE(0x01));
   dacc_set_channel_selection(DACC_INTERFACE, channel);
   if ((dacc_get_channel_status(DACC_INTERFACE) & (1 << channel)) == 0) 
				dacc_enable_channel(DACC_INTERFACE, channel);
}
                
void config_double_dac() {
   pmc_enable_periph_clk(DACC_INTERFACE_ID);
   dacc_reset(DACC_INTERFACE);
   dacc_set_transfer_mode(DACC_INTERFACE, 1); //  word transfert (32 bits)
   dacc_set_power_save(DACC_INTERFACE, 0, 0);
   dacc_set_timing(DACC_INTERFACE, 0x08, 0, 0x10);
   dacc_set_analog_control(DACC_INTERFACE, DACC_ACR_IBCTLCH0(0x02)|DACC_ACR_IBCTLCH1(0x02)|DACC_ACR_IBCTLDACCORE(0x01));
   DACC_INTERFACE->DACC_MR |= DACC_MR_TAG;
   DACC_INTERFACE->DACC_CHER = 0x3;
   chsel = (1<<13) | (1<<28);
}
                 
volatile void adc_copie_une_voie() {
     DACC_INTERFACE->DACC_CDR = *(ADC->ADC_CDR+channels[0]);  
}

volatile void adc_copie_deux_voies() {
     
     DACC_INTERFACE->DACC_CDR = *(ADC->ADC_CDR+channels[0]) | (*(ADC->ADC_CDR+channels[1]) << 16) | chsel;  
}
                    
volatile void adc_filtrage_float_FD1_une_voie() {
   float accum_0;
   int16_t sortie_0;
   uint8_t i,k;
   x_0[J_tampon] = ((int16_t)*(ADC->ADC_CDR+channels[0]))-offset;
   accum_0 = 0;
   i = J_tampon;
   for (k=0; k<nb; k++) {
     accum_0 += x_0[i]*bbb[k];
     i--;  
   }
   i = J_tampon-1;
   for (k=1; k<na; k++) {
     accum_0 -= y_0[i]*aaa[k];
     i--;  
   }
   sortie_0 = accum_0;
   y_0[J_tampon] = sortie_0;
   J_tampon++;
   sortie_0 = sortie_0*GAIN+offset;
#ifdef SAT
   if (sortie_0<0) sortie_0 = 0;
   else if (sortie_0 > 0xFFF) sortie_0 = 0xFFF;
#endif
   DACC_INTERFACE->DACC_CDR = sortie_0;
}
                   
volatile void adc_filtrage_float_FD1_deux_voies() {
   float accum_0,accum_1;
   int16_t sortie_0, sortie_1;
   uint8_t i,k;
   x_0[J_tampon] = (int16_t)(*(ADC->ADC_CDR+channels[0]))-offset;
   x_1[J_tampon] = (int16_t)(*(ADC->ADC_CDR+channels[1]))-offset;
   accum_0 = 0;
   accum_1 = 0;
   i = J_tampon;
   for (k=0; k<nb; k++) {
     accum_0 += x_0[i]*bbb[k];
     accum_1 += x_1[i]*bbb[k];
     i--;  
   }
   i = J_tampon-1;
   for (k=1; k<na; k++) {
     accum_0 -= y_0[i]*aaa[k];
     accum_1 -= y_1[i]*aaa[k];
     i--;  
   }
   sortie_0 = accum_0;
   sortie_1 = accum_1;
   y_0[J_tampon] = sortie_0;
   y_1[J_tampon] = sortie_1;
   J_tampon++;
   sortie_0 = sortie_0*GAIN+offset;
   sortie_1 = sortie_1*GAIN+offset;
#ifdef SAT
   if (sortie_0<0) sortie_0 = 0;
   else if (sortie_0 > 0xFFF) sortie_0 = 0xFFF;
   if (sortie_1<0) sortie_1 = 0;
   else if (sortie_1 > 0xFFF) sortie_1 = 0xFFF;
#endif
   DACC_INTERFACE->DACC_CDR = (sortie_0) | (((uint32_t)(sortie_1)) << 16) | chsel;;
}
                     
volatile void adc_filtrage_float_FD2_une_voie() {
  float accum_0;
  int16_t sortie_0;
  uint8_t i,k;
  accum_0 = ((int16_t)(*(ADC->ADC_CDR+channels[0]))-offset)*aaa[0];
  i = J_tampon-1;
  for (k=1; k<na; k++) {
    accum_0 -= www_0[i]*aaa[k];
    i--;  
  }
  www_0[J_tampon] = accum_0;
  accum_0 = 0.0;
  i = J_tampon;
  for (k=0; k<nb; k++) {
      accum_0 += www_0[i]*bbb[k];
      i--;  
  }
  J_tampon++;
   sortie_0 = accum_0*GAIN+offset;
#ifdef SAT
   if (sortie_0<0) sortie_0 = 0;
   else if (sortie_0 > 0xFFF) sortie_0 = 0xFFF;
#endif
   DACC_INTERFACE->DACC_CDR = sortie_0;
}

volatile void adc_filtrage_float_FD2_deux_voies() {
  float accum_0,accum_1;
  int16_t sortie_0,sortie_1;
  uint8_t i,k;
  accum_0 = ((int16_t)(*(ADC->ADC_CDR+channels[0]))-offset)*aaa[0];
  accum_1 = ((int16_t)(*(ADC->ADC_CDR+channels[1]))-offset)*aaa[0];
  i = J_tampon-1;
  for (k=1; k<na; k++) {
    accum_0 -= www_0[i]*aaa[k];
    accum_1 -= www_1[i]*aaa[k];
    i--;  
  }
  www_0[J_tampon] = accum_0;
  www_1[J_tampon] = accum_1;
  accum_0 = 0.0;
  accum_1 = 0.0;
  i = J_tampon;
  for (k=0; k<nb; k++) {
      accum_0 += www_0[i]*bbb[k];
      accum_1 += www_1[i]*bbb[k];
      i--;  
  }
  J_tampon++;
   sortie_0 = accum_0*GAIN+offset;
   sortie_1 = accum_1*GAIN+offset;
#ifdef SAT
   if (sortie_0<0) sortie_0 = 0;
   else if (sortie_0 > 0xFFF) sortie_0 = 0xFFF;
   if (sortie_1<0) sortie_1 = 0;
   else if (sortie_1 > 0xFFF) sortie_1 = 0xFFF;
#endif
   DACC_INTERFACE->DACC_CDR = (sortie_0) | (((uint32_t)(sortie_1)) << 16) | chsel;;
}
                     
volatile void adc_filtrage_float_FDT2_une_voie() {
  int16_t entree_0,sortie_0;
  uint8_t k,i;
  i = 0;
  entree_0 = (int16_t)(*(ADC->ADC_CDR+channels[0]))-offset;
  sortie_0 = bbb[0]*entree_0+www_0[0];
  for (k=1; k<max_na_nb; k++) {
      www_0[i] = bbb[k]*entree_0 - aaa[k]*sortie_0 + www_0[i+1];
      i++; 
  }
   sortie_0 = sortie_0*GAIN+offset;
#ifdef SAT
   if (sortie_0<0) sortie_0 = 0;
   else if (sortie_0 > 0xFFF) sortie_0 = 0xFFF;
#endif
   DACC_INTERFACE->DACC_CDR = sortie_0;
}

volatile void adc_filtrage_float_FDT2_deux_voies() {
  int16_t entree_0,sortie_0,entree_1,sortie_1;
  uint8_t k,i;
  i = 0;
  entree_0 = (int16_t)(*(ADC->ADC_CDR+channels[0]))-offset;
  entree_1 = (int16_t)(*(ADC->ADC_CDR+channels[1]))-offset;
  sortie_0 = bbb[0]*entree_0+www_0[0];
  sortie_1 = bbb[0]*entree_1+www_1[0];
  for (k=1; k<max_na_nb; k++) {
      www_0[i] = bbb[k]*entree_0 - aaa[k]*sortie_0 + www_0[i+1];
      www_1[i] = bbb[k]*entree_1 - aaa[k]*sortie_1 + www_1[i+1];
      i++; 
  }
   sortie_0 = sortie_0*GAIN+offset;
   sortie_1 = sortie_1*GAIN+offset;
#ifdef SAT
   if (sortie_0<0) sortie_0 = 0;
   else if (sortie_0 > 0xFFF) sortie_0 = 0xFFF;
   if (sortie_1<0) sortie_1 = 0;
   else if (sortie_1 > 0xFFF) sortie_1 = 0xFFF;
#endif
   DACC_INTERFACE->DACC_CDR = (sortie_0) | (((uint32_t)(sortie_1)) << 16) | chsel;;
}
                      
volatile void adc_filtrage_32bits_FD1_une_voie() {
   int32_t accum_0;
   int16_t sortie_0;
   uint8_t i,k;
   x_0[J_tampon] = (int16_t)(*(ADC->ADC_CDR+channels[0]))-offset;
   accum_0 = 0;
   i = J_tampon;
   for (k=0; k<nb; k++) {
      accum_0 += x_0[i]*bb[k];
      i--;
   }
   i = J_tampon-1;
   for (k=1; k<na; k++) {
      accum_0 -= y_0[i]*aa[k];
      i--;
   }
   sortie_0 = accum_0 >> ab_shift;
   y_0[J_tampon] = sortie_0;
   J_tampon++;
   sortie_0 = sortie_0*GAIN+offset;
#ifdef SAT
   if (sortie_0<0) sortie_0 = 0;
   else if (sortie_0 > 0xFFF) sortie_0 = 0xFFF;
#endif
   DACC_INTERFACE->DACC_CDR = sortie_0;
}

volatile void adc_filtrage_32bits_FD1_deux_voies() {
   int32_t accum_0, accum_1;
   int16_t sortie_0, sortie_1;
   uint8_t i,k;
   x_0[J_tampon] = (int16_t)(*(ADC->ADC_CDR+channels[0]))-offset;
   x_1[J_tampon] = (int16_t)(*(ADC->ADC_CDR+channels[1]))-offset;
   accum_0 = 0;
   accum_1 = 0;
   i = J_tampon;
   for (k=0; k<nb; k++) {
      accum_0 += x_0[i]*bb[k];
      accum_1 += x_1[i]*bb[k];
      i--;
   }
   i = J_tampon-1;
   for (k=1; k<na; k++) {
      accum_0 -= y_0[i]*aa[k];
      accum_1 -= y_1[i]*aa[k];
      i--;
   }
   sortie_0 = accum_0 >> ab_shift;
   sortie_1 = accum_1 >> ab_shift;
   y_0[J_tampon] = sortie_0;
   y_1[J_tampon] = sortie_1;
   J_tampon++;
   sortie_0 = sortie_0*GAIN+offset;
   sortie_1 = sortie_1*GAIN+offset;
#ifdef SAT
   if (sortie_0<0) sortie_0 = 0;
   else if (sortie_0 > 0xFFF) sortie_0 = 0xFFF;
   if (sortie_1<0) sortie_1 = 0;
   else if (sortie_1 > 0xFFF) sortie_1 = 0xFFF;
#endif
   DACC_INTERFACE->DACC_CDR = (sortie_0) | (((uint32_t)(sortie_1)) << 16) | chsel;;
}
                   
volatile void adc_filtrage_32bits_FD2_une_voie() {
   int32_t accum_0;
   int16_t sortie_0;
    uint8_t i,k;
    accum_0 =  ((int16_t)(*(ADC->ADC_CDR+channels[0]))-offset)*aa[0];
    i = J_tampon-1;
    for (k=1; k<na; k++) {
      accum_0 -= ww_0[i]*aa[k];
      i--; 
    }
    ww_0[J_tampon] = accum_0 >> ab_shift;
    i = J_tampon;
    accum_0 = 0;
    for (k=0; k<nb; k++) {
      accum_0 += ww_0[i]*bb[k];
      i--;  
    }
    J_tampon++;
    sortie_0 = (accum_0 >> ab_shift)*GAIN+offset;
#ifdef SAT
   if (sortie_0<0) sortie_0 = 0;
   else if (sortie_0 > 0xFFF) sortie_0 = 0xFFF;
#endif
   DACC_INTERFACE->DACC_CDR = sortie_0;
}

volatile void adc_filtrage_32bits_FD2_deux_voies() {
   int32_t accum_0,accum_1;
   int16_t sortie_0,sortie_1;
    uint8_t i,k;
    accum_0 =  ((int16_t)(*(ADC->ADC_CDR+channels[0]))-offset)*aa[0];
    accum_1 =  ((int16_t)(*(ADC->ADC_CDR+channels[1]))-offset)*aa[0];
    i = J_tampon-1;
    for (k=1; k<na; k++) {
      accum_0 -= ww_0[i]*aa[k];
      accum_1 -= ww_1[i]*aa[k];
      i--; 
    }
    ww_0[J_tampon] = accum_0 >> ab_shift;
    ww_1[J_tampon] = accum_1 >> ab_shift;
    i = J_tampon;
    accum_0 = 0;
    accum_1 = 0;
    for (k=0; k<nb; k++) {
      accum_0 += ww_0[i]*bb[k];
      accum_1 += ww_1[i]*bb[k];
      i--;  
    }
    J_tampon++;
   sortie_0 = (accum_0 >> ab_shift)*GAIN+offset;
   sortie_1 = (accum_1 >> ab_shift)*GAIN+offset;
#ifdef SAT
   if (sortie_0<0) sortie_0 = 0;
   else if (sortie_0 > 0xFFF) sortie_0 = 0xFFF;
   if (sortie_1<0) sortie_1 = 0;
   else if (sortie_1 > 0xFFF) sortie_1 = 0xFFF;
#endif
   DACC_INTERFACE->DACC_CDR = (sortie_0) | (((uint32_t)(sortie_1)) << 16) | chsel;;
}
                   
volatile void adc_filtrage_32bits_FDT2_une_voie() {
   int32_t entree_0, sortie_0;
   uint8_t k,i;
   i = 0;
   entree_0 = (int16_t)(*(ADC->ADC_CDR+channels[0]))-offset;
   sortie_0 = (bb[0]*entree_0+ww_0[0]) >> ab_shift;
   for (k=1; k<max_na_nb; k++) {
       ww_0[i] = entree_0*bb[k] - sortie_0*aa[k] + ww_0[i+1];
       i++;
   }
   sortie_0 = sortie_0*GAIN+offset;
#ifdef SAT
   if (sortie_0<0) sortie_0 = 0;
   else if (sortie_0 > 0xFFF) sortie_0 = 0xFFF;
#endif
   DACC_INTERFACE->DACC_CDR = sortie_0;
}

volatile void adc_filtrage_32bits_FDT2_deux_voies() {
   int32_t entree_0, sortie_0, entree_1, sortie_1;
   uint8_t k,i;
   i = 0;
   entree_0 = (int16_t)(*(ADC->ADC_CDR+channels[0]))-offset;
   entree_1 = (int16_t)(*(ADC->ADC_CDR+channels[1]))-offset;
   sortie_0 = (bb[0]*entree_0+ww_0[0]) >> ab_shift;
   sortie_1 = (bb[0]*entree_1+ww_1[0]) >> ab_shift;
   for (k=1; k<max_na_nb; k++) {
       ww_0[i] = entree_0*bb[k] - sortie_0*aa[k] + ww_0[i+1];
       ww_1[i] = entree_1*bb[k] - sortie_1*aa[k] + ww_1[i+1];
       i++;
   }
   sortie_0 = sortie_0*GAIN+offset;
   sortie_1 = sortie_1*GAIN+offset;
#ifdef SAT
   if (sortie_0<0) sortie_0 = 0;
   else if (sortie_0 > 0xFFF) sortie_0 = 0xFFF;
   if (sortie_1<0) sortie_1 = 0;
   else if (sortie_1 > 0xFFF) sortie_1 = 0xFFF;
#endif
   DACC_INTERFACE->DACC_CDR = (sortie_0) | (((uint32_t)(sortie_1)) << 16) | chsel;;
}
                   
void init_tampons() {
   int i;
   J_tampon = 0;
   for (i=0; i<TAILLE_TAMPON; i++) {
     x_0[i] = 0;
     y_0[i] = 0;
     x_1[i] = 0;
     y_1[i] = 0; 
     www_0[i] = 0.0;
     www_1[i] = 0.0;
     ww_0[i] = 0;
     ww_1[i] = 0;
   } 
}
                   
void lecture_config_adc() {
   uint32_t c1,c2,c3,c4;
   int k;
   while (Serial.available()<1) {};
    nchannels = Serial.read();
    while (Serial.available()<nchannels) {};
    for (k=0; k<nchannels; k++) {
      if (k < MAX_NCHAN) channels[k] = channels_pins[Serial.read()]; 
    }
    while (Serial.available()<nchannels) {};
    for (k=0; k<nchannels; k++) {
      if (k < MAX_NCHAN) gain[k] = Serial.read(); 
    }
    if (nchannels > MAX_NCHAN) nchannels = MAX_NCHAN;
    while (Serial.available()<4) {};
    c1 = Serial.read();
    c2 = Serial.read();
    c3 = Serial.read();
    c4 = Serial.read();
    ticks = ((c1 << 24) | (c2 << 16) | (c3 << 8) | c4);
}
                  
void lecture_filtrage_float() {
  uint32_t c1,c2,c3,c4;
  int k;
  int32_t m;
  int8_t e;
  uint8_t type;
  lecture_config_adc();
    
    // filtrage
   while (Serial.available()<1) {};
   na = Serial.read();
   for (int k=0; k<na; k++) {
       while (Serial.available()<5) {};
       c1 = Serial.read();
       c2 = Serial.read();
       c3 = Serial.read();
       c4 = Serial.read();
       e = Serial.read();
       m = ((c1 << 24) | (c2 << 16) | (c3 << 8) | c4);
       aaa[k] = ((float)m)*pow(2,e-30);
   }
   while (Serial.available()<1) {};
   nb = Serial.read();
   for (int k=0; k<nb; k++) {
       while (Serial.available()<5) {};
       c1 = Serial.read();
       c2 = Serial.read();
       c3 = Serial.read();
       c4 = Serial.read();
       e = Serial.read();
       m = ((c1 << 24) | (c2 << 16) | (c3 << 8) | c4);
       bbb[k] = ((float)m)*pow(2,e-30);
   }
   while (Serial.available()<2) {};
   c1 = Serial.read();
   c2 = Serial.read();
   offset = ((c1 << 8) | c2);
   while (Serial.available()<1) {};
   type = Serial.read();
 
   init_tampons();
   max_na_nb = max(na,nb);
   for (k=na; k<max_na_nb; k++) aaa[k] = 0.0;
   for (k=nb; k<max_na_nb; k++) bbb[k] = 0.0;
   if (nchannels==1) {
     config_dac(0);
     if (type==1) ADC_Handler_function = adc_filtrage_float_FD1_une_voie;
     else if (type==2) ADC_Handler_function = adc_filtrage_float_FD2_une_voie;
     else if (type==3) ADC_Handler_function = adc_filtrage_float_FDT2_une_voie;
     else ADC_Handler_function = adc_copie_une_voie;
   }
   else if (nchannels==2) {
     config_double_dac();
     if (type==1) ADC_Handler_function = adc_filtrage_float_FD1_deux_voies;
     else if (type==2) ADC_Handler_function = adc_filtrage_float_FD2_deux_voies;
     else if (type==3) ADC_Handler_function = adc_filtrage_float_FDT2_deux_voies;
     else ADC_Handler_function = adc_copie_deux_voies;
   }
   config_adc();
   config_timer(ticks);
}
                   
void lecture_filtrage_32bits() {
  uint32_t c1,c2,c3,c4;
  int k;
  uint8_t type;
  lecture_config_adc();
  while (Serial.available()<1) {};
   na = Serial.read();
   for (int k=0; k<na; k++) {
       while (Serial.available()<4) {};
       c1 = Serial.read();
       c2 = Serial.read();
       c3 = Serial.read();
       c4 = Serial.read();
       aa[k] = ((c1 << 24) | (c2 << 16) | (c3 << 8) | c4);
   }
   while (Serial.available()<1) {};
   nb = Serial.read();
   for (int k=0; k<nb; k++) {
       while (Serial.available()<4) {};
       c1 = Serial.read();
       c2 = Serial.read();
       c3 = Serial.read();
       c4 = Serial.read();
       bb[k] = ((c1 << 24) | (c2 << 16) | (c3 << 8) | c4);
   }
   while (Serial.available()<1) {};
   ab_shift = Serial.read();
   while (Serial.available()<2) {};
   c1 = Serial.read();
   c2 = Serial.read();
   offset = ((c1 << 8) | c2);
   while (Serial.available()<1) {};
   type = Serial.read();
   init_tampons();
   max_na_nb = max(na,nb);
   for (k=na; k<max_na_nb; k++) aa[k] = 0.0;
   for (k=nb; k<max_na_nb; k++) bb[k] = 0.0;
   if (nchannels==1) {
     config_dac(0);
     if (type==1) ADC_Handler_function = adc_filtrage_32bits_FD1_une_voie;
     else if (type==2) ADC_Handler_function = adc_filtrage_32bits_FD2_une_voie;
     else if (type==3) ADC_Handler_function = adc_filtrage_32bits_FDT2_une_voie;
     else ADC_Handler_function = adc_copie_une_voie;
   }
   else if (nchannels==2) {
     config_double_dac();
     if (type==1) ADC_Handler_function = adc_filtrage_32bits_FD1_deux_voies;
     else if (type==2) ADC_Handler_function = adc_filtrage_32bits_FD2_deux_voies;
     else if (type==3) ADC_Handler_function = adc_filtrage_32bits_FDT2_deux_voies;
     else ADC_Handler_function = adc_copie_deux_voies;
   }
   config_adc();
   config_timer(ticks);
}
                    
void setup() {
  Serial.begin(115200);
  pinMode(3,OUTPUT);
}
                    
void lecture_serie() {
  char com;
    if (Serial.available()>0) {
        com = Serial.read();
        if (com==SET_FILTRAGE_FLOAT) lecture_filtrage_float();
        else if (com==SET_FILTRAGE_32BITS) lecture_filtrage_32bits();
        else if (com==IS_READY) {
          stop_timer();
          Serial.write("O");  
        }
    }
}
                     
void loop() {
  lecture_serie();
  delay(500);
}
                     